/*
    SPDX-FileCopyrightText: 2015 Andreas Cord-Landwehr <cordlandwehr@kde.org>

    SPDX-License-Identifier: LGPL-2.1-or-later
*/

#include "artikulate_debug.h"
Q_LOGGING_CATEGORY(ARTIKULATE_LOG, "log_artikulate")
Q_LOGGING_CATEGORY(ARTIKULATE_CORE, "articulate.core")
Q_LOGGING_CATEGORY(ARTIKULATE_PARSER, "articulate.parser")
